@@ -77,7 +77,11 @@ fn white_space(input: &[u8]) -> NomResult<()> {
 }
 
 fn space(input: &[u8]) -> NomResult<()> {
-    fold_many0(alt((map(take_while1(is_whitespace), |_| ()), comment)), (), |_, _| ())(input)
+    fold_many0(
+        alt((map(take_while1(is_whitespace), |_| ()), comment)),
+        || {},
+        |_, _| (),
+    )(input)
 }
 
 fn integer(input: &[u8]) -> NomResult<i64> {
@@ -147,19 +151,19 @@ fn escape_sequence(input: &[u8]) -> NomResult<Option<u8>> {
     )(input)
 }
 
-enum ILS<'a> {
+enum InnerLiteralString<'a> {
     Direct(&'a [u8]),
     Escape(Option<u8>),
     Eol(&'a [u8]),
     Nested(Vec<u8>),
 }
 
-impl<'a> ILS<'a> {
+impl<'a> InnerLiteralString<'a> {
     fn push(&self, output: &mut Vec<u8>) {
         match self {
-            ILS::Direct(s) | ILS::Eol(s) => output.extend_from_slice(s),
-            ILS::Escape(e) => output.extend(e),
-            ILS::Nested(n) => output.extend_from_slice(n),
+            InnerLiteralString::Direct(s) | InnerLiteralString::Eol(s) => output.extend_from_slice(s),
+            InnerLiteralString::Escape(e) => output.extend(e),
+            InnerLiteralString::Nested(n) => output.extend_from_slice(n),
         }
     }
 }
@@ -168,12 +172,12 @@ fn inner_literal_string(depth: usize) -> impl Fn(&[u8]) -> NomResult<Vec<u8>> {
     move |input| {
         fold_many0(
             alt((
-                map(take_while1(is_direct_literal_string), ILS::Direct),
-                map(escape_sequence, ILS::Escape),
-                map(eol, ILS::Eol),
-                map(nested_literal_string(depth), ILS::Nested),
+                map(take_while1(is_direct_literal_string), InnerLiteralString::Direct),
+                map(escape_sequence, InnerLiteralString::Escape),
+                map(eol, InnerLiteralString::Eol),
+                map(nested_literal_string(depth), InnerLiteralString::Nested),
             )),
-            Vec::new(),
+            Vec::new,
             |mut out: Vec<u8>, value| {
                 value.push(&mut out);
                 out
@@ -217,7 +221,7 @@ fn hexadecimal_string(input: &[u8]) -> NomResult<Object> {
             terminated(
                 fold_many0(
                     preceded(white_space, hex_digit),
-                    (Vec::new(), false),
+                    || -> (Vec<u8>, bool) { (Vec::new(), false) },
                     |state, c| match state {
                         (mut out, false) => {
                             out.push(c << 4);
@@ -257,7 +261,7 @@ fn dictionary(input: &[u8]) -> NomResult<Dictionary> {
         pair(tag(b"<<"), space),
         fold_many0(
             pair(terminated(name, space), _direct_object),
-            Dictionary::new(),
+            Dictionary::new,
             |mut dict, (key, value)| {
                 dict.set(key, value);
                 dict
@@ -327,8 +331,8 @@ fn object<'a>(input: &'a [u8], reader: &Reader) -> NomResult<'a, Object> {
     terminated(alt((|input| stream(input, reader), _direct_objects)), space)(input)
 }
 
-pub fn indirect_object<'a>(
-    input: &'a [u8], offset: usize, expected_id: Option<ObjectId>, reader: &Reader,
+pub fn indirect_object(
+    input: &[u8], offset: usize, expected_id: Option<ObjectId>, reader: &Reader,
 ) -> crate::Result<(ObjectId, Object)> {
     let (id, mut object) = _indirect_object(&input[offset..], offset, expected_id, reader)?;
 
@@ -337,8 +341,8 @@ pub fn indirect_object<'a>(
     Ok((id, object))
 }
 
-fn _indirect_object<'a>(
-    input: &'a [u8], offset: usize, expected_id: Option<ObjectId>, reader: &Reader,
+fn _indirect_object(
+    input: &[u8], offset: usize, expected_id: Option<ObjectId>, reader: &Reader,
 ) -> crate::Result<(ObjectId, Object)> {
     let (i, object_id) = terminated(object_id, pair(tag(b"obj"), space))(input).map_err(|_| Error::Parse { offset })?;
     if let Some(expected_id) = expected_id {
@@ -384,7 +388,7 @@ fn xref(input: &[u8]) -> NomResult<Xref> {
         pair(tag(b"xref"), eol),
         fold_many1(
             xref_section,
-            Xref::new(0, XrefType::CrossReferenceTable),
+            || -> Xref { Xref::new(0, XrefType::CrossReferenceTable) },
             |mut xref, ((start, _count), entries)| {
                 for (index, ((offset, generation), is_normal)) in entries.into_iter().enumerate() {
                     if is_normal {
