@@ -1,6 +1,7 @@
 use crate::{Document, Error, Result};
 use linked_hash_map::{self, Iter, IterMut, LinkedHashMap};
 use log::warn;
+use std::cmp::max;
 use std::fmt;
 use std::str;
 
@@ -33,7 +34,7 @@ pub enum Object {
     Null,
     Boolean(bool),
     Integer(i64),
-    Real(f64),
+    Real(f32),
     Name(Vec<u8>),
     String(Vec<u8>, StringFormat),
     Array(Vec<Object>),
@@ -43,18 +44,13 @@ pub enum Object {
 }
 
 /// String objects can be written in two formats.
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Default)]
 pub enum StringFormat {
+    #[default]
     Literal,
     Hexadecimal,
 }
 
-impl Default for StringFormat {
-    fn default() -> StringFormat {
-        StringFormat::Literal
-    }
-}
-
 impl From<bool> for Object {
     fn from(value: bool) -> Self {
         Object::Boolean(value)
@@ -86,13 +82,13 @@ from_smaller_ints! {
 
 impl From<f64> for Object {
     fn from(number: f64) -> Self {
-        Object::Real(number)
+        Object::Real(number as f32)
     }
 }
 
 impl From<f32> for Object {
     fn from(number: f32) -> Self {
-        Object::Real(f64::from(number))
+        Object::Real(number)
     }
 }
 
@@ -115,8 +111,8 @@ impl From<Vec<Object>> for Object {
 }
 
 impl From<Dictionary> for Object {
-    fn from(dcit: Dictionary) -> Self {
-        Object::Dictionary(dcit)
+    fn from(dict: Dictionary) -> Self {
+        Object::Dictionary(dict)
     }
 }
 
@@ -155,7 +151,7 @@ impl Object {
         }
     }
 
-    pub fn as_f64(&self) -> Result<f64> {
+    pub fn as_f32(&self) -> Result<f32> {
         match *self {
             Object::Real(ref value) => Ok(*value),
             _ => Err(Error::Type),
@@ -164,9 +160,9 @@ impl Object {
 
     /// Get the object value as a float.
     /// Unlike as_f64() this will also cast an Integer to a Real.
-    pub fn as_float(&self) -> Result<f64> {
+    pub fn as_float(&self) -> Result<f32> {
         match *self {
-            Object::Integer(ref value) => Ok(*value as f64),
+            Object::Integer(ref value) => Ok(*value as f32),
             Object::Real(ref value) => Ok(*value),
             _ => Err(Error::Type),
         }
@@ -197,6 +193,13 @@ impl Object {
         }
     }
 
+    pub fn as_string(&self) -> Result<std::borrow::Cow<'_, str>> {
+        match self {
+            Object::String(string, _) => Ok(std::string::String::from_utf8_lossy(string)),
+            _ => Err(Error::Type),
+        }
+    }
+
     pub fn as_reference(&self) -> Result<ObjectId> {
         match *self {
             Object::Reference(ref id) => Ok(*id),
@@ -351,86 +354,66 @@ impl Dictionary {
     pub fn extend(&mut self, other: &Dictionary) {
         let keep_both_objects =
             |new_dict: &mut LinkedHashMap<Vec<u8>, Object>, key: &Vec<u8>, value: &Object, old_value: &Object| {
-                let mut final_array = Vec::new();
+                let mut final_array;
+
                 match value {
                     Object::Array(array) => {
+                        final_array = Vec::with_capacity(array.len() + 1);
                         final_array.push(old_value.to_owned());
                         final_array.extend(array.to_owned());
                     }
                     _ => {
-                        final_array.push(value.to_owned());
-                        final_array.push(old_value.to_owned());
+                        final_array = vec![value.to_owned(), old_value.to_owned()];
                     }
                 }
 
                 new_dict.insert(key.to_owned(), Object::Array(final_array));
             };
 
-        let mut new_dict = LinkedHashMap::new();
+        let mut new_dict = LinkedHashMap::with_capacity(other.0.len() + 1);
+
         for (key, value) in other.0.iter() {
             if let Some(old_value) = self.0.get(key) {
-                match old_value {
-                    Object::Dictionary(old_dict) => match value {
-                        Object::Dictionary(dict) => {
-                            let mut replaced_dict = old_dict.to_owned();
-                            replaced_dict.extend(dict);
+                match (old_value, value) {
+                    (Object::Dictionary(old_dict), Object::Dictionary(dict)) => {
+                        let mut replaced_dict = old_dict.to_owned();
+                        replaced_dict.extend(dict);
 
-                            new_dict.insert(key.to_owned(), Object::Dictionary(replaced_dict));
-                        }
-                        _ => keep_both_objects(&mut new_dict, key, value, old_value),
-                    },
-                    Object::Array(old_array) => match value {
-                        Object::Array(array) => {
-                            let mut replaced_array = old_array.to_owned();
-                            replaced_array.extend(array.to_owned());
+                        new_dict.insert(key.to_owned(), Object::Dictionary(replaced_dict));
+                    }
+                    (Object::Array(old_array), Object::Array(array)) => {
+                        let mut replaced_array = old_array.to_owned();
+                        replaced_array.extend(array.to_owned());
 
-                            new_dict.insert(key.to_owned(), Object::Array(replaced_array));
-                        }
-                        _ => keep_both_objects(&mut new_dict, key, value, old_value),
-                    },
-                    Object::Integer(old_id) => match value {
-                        Object::Integer(id) => {
-                            let mut array = Vec::new();
-                            array.push(Object::Integer(*old_id));
-                            array.push(Object::Integer(*id));
+                        new_dict.insert(key.to_owned(), Object::Array(replaced_array));
+                    }
+                    (Object::Integer(old_id), Object::Integer(id)) => {
+                        let array = vec![Object::Integer(*old_id), Object::Integer(*id)];
 
-                            new_dict.insert(key.to_owned(), Object::Array(array));
-                        }
-                        _ => keep_both_objects(&mut new_dict, key, value, old_value),
-                    },
-                    Object::Real(old_id) => match value {
-                        Object::Real(id) => {
-                            let mut array = Vec::new();
-                            array.push(Object::Real(*old_id));
-                            array.push(Object::Real(*id));
+                        new_dict.insert(key.to_owned(), Object::Array(array));
+                    }
+                    (Object::Real(old_id), Object::Real(id)) => {
+                        let array = vec![Object::Real(*old_id), Object::Real(*id)];
 
-                            new_dict.insert(key.to_owned(), Object::Array(array));
-                        }
-                        _ => keep_both_objects(&mut new_dict, key, value, old_value),
-                    },
-                    Object::String(old_ids, old_format) => match value {
-                        Object::String(ids, format) => {
-                            let mut array = Vec::new();
-                            array.push(Object::String(old_ids.to_owned(), old_format.to_owned()));
-                            array.push(Object::String(ids.to_owned(), format.to_owned()));
+                        new_dict.insert(key.to_owned(), Object::Array(array));
+                    }
+                    (Object::String(old_ids, old_format), Object::String(ids, format)) => {
+                        let array = vec![
+                            Object::String(old_ids.to_owned(), old_format.to_owned()),
+                            Object::String(ids.to_owned(), format.to_owned()),
+                        ];
 
-                            new_dict.insert(key.to_owned(), Object::Array(array));
-                        }
-                        _ => keep_both_objects(&mut new_dict, key, value, old_value),
-                    },
-                    Object::Reference(old_object_id) => match value {
-                        Object::Reference(object_id) => {
-                            let mut array = Vec::new();
-                            array.push(Object::Reference(*old_object_id));
-                            array.push(Object::Reference(*object_id));
+                        new_dict.insert(key.to_owned(), Object::Array(array));
+                    }
+                    (Object::Reference(old_object_id), Object::Reference(object_id)) => {
+                        let array = vec![Object::Reference(*old_object_id), Object::Reference(*object_id)];
 
-                            new_dict.insert(key.to_owned(), Object::Array(array));
-                        }
-                        _ => keep_both_objects(&mut new_dict, key, value, old_value),
-                    },
-                    _ => {
+                        new_dict.insert(key.to_owned(), Object::Array(array));
+                    }
+                    (Object::Null, _) | (Object::Boolean(_), _) | (Object::Name(_), _) | (Object::Stream(_), _) => {
                         new_dict.insert(key.to_owned(), old_value.to_owned());
                     }
+                    (_, _) => keep_both_objects(&mut new_dict, key, value, old_value),
                 }
             } else {
                 new_dict.insert(key.to_owned(), value.to_owned());
@@ -439,6 +422,17 @@ impl Dictionary {
 
         self.0 = new_dict;
     }
+
+    /// Return a reference to the inner  [`LinkedHashMap`].
+    pub fn as_hashmap(&self) -> &LinkedHashMap<Vec<u8>, Object> {
+        &self.0
+    }
+
+
+    /// Return a mut reference to the inner [`LinkedHashMap`].
+    pub fn as_hashmap_mut(&mut self) -> &mut LinkedHashMap<Vec<u8>, Object> {
+        &mut self.0
+    }
 }
 
 #[macro_export]
@@ -597,7 +591,7 @@ impl Stream {
     }
 
     fn decompress_lzw(input: &[u8], params: Option<&Dictionary>) -> Result<Vec<u8>> {
-        use weezl::{BitOrder, decode::Decoder};
+        use weezl::{decode::Decoder, BitOrder};
         const MIN_BITS: u8 = 9;
 
         let early_change = params
@@ -649,9 +643,9 @@ impl Stream {
         if let Some(params) = params {
             let predictor = params.get(b"Predictor").and_then(Object::as_i64).unwrap_or(1);
             if (10..=15).contains(&predictor) {
-                let pixels_per_row = params.get(b"Columns").and_then(Object::as_i64).unwrap_or(1) as usize;
-                let colors = params.get(b"Colors").and_then(Object::as_i64).unwrap_or(1) as usize;
-                let bits = params.get(b"BitsPerComponent").and_then(Object::as_i64).unwrap_or(8) as usize;
+                let pixels_per_row = max(1, params.get(b"Columns").and_then(Object::as_i64).unwrap_or(1)) as usize;
+                let colors = max(1, params.get(b"Colors").and_then(Object::as_i64).unwrap_or(1)) as usize;
+                let bits = max(8, params.get(b"BitsPerComponent").and_then(Object::as_i64).unwrap_or(8)) as usize;
                 let bytes_per_pixel = colors * bits / 8;
                 data = png::decode_frame(data.as_slice(), bytes_per_pixel, pixels_per_row)?;
             }
